const { Pool } = require('pg');

const dbConfig = {
    user: process.env.POSTGRES_USER || 'e2ee',
    password: process.env.POSTGRES_PASSWORD,
    host: process.env.POSTGRES_HOST || '127.0.0.1',
    port: process.env.POSTGRES_PORT || '5432',
    database: process.env.POSTGRES_DB || 'e2ee',
};

const pool = new Pool(dbConfig);

const database = {
    init: async () => {
        while (true) {
            try {
                await database.checkIfDatabaseIsUp();
                console.log('Database is up, initializing...');
                database.checkSchema().then((exists) => {
                    if (!exists) {
                        database.createTables();
                    } else {
                        console.log("Database schema is already correct.");
                    }
                }).catch((err) => {
                    console.error("Error checking schema:", err);
                });
                break;
            } catch (err) {
                console.error('Waiting for the database to start...');
            }
            await new Promise(resolve => setTimeout(resolve, 5000));
        }
    },
    createTables: () => {
        pool.query(`
            CREATE TABLE IF NOT EXISTS "user" (
                uuid integer PRIMARY KEY,
                pubkey text
            );
        `, (err, _) => {
            if (err) {
                console.error('Error creating user table', err);
                return;
            }
            pool.query(`
                CREATE SEQUENCE IF NOT EXISTS uuid_sequence
                INCREMENT BY 1
                START WITH 1;
            `, (err, _) => {
                if (err) {
                    console.error('Error creating sequence', err);
                } else {
                    console.log("user table and sequence created successfully.");
                }
            });
        });
    },
    checkSchema: () => {
        return new Promise((resolve, reject) => {
            pool.query(`
                SELECT EXISTS (
                    SELECT 1 FROM pg_tables WHERE tablename = 'user'
                ) AS table_exists;
            `, (err, res) => {
                if (err) {
                    console.error('Error executing query', err);
                    reject(err);
                } else {
                    const tableExists = res.rows[0].table_exists;
                    resolve(tableExists);
                }
            });
        });
    },
    checkIfDatabaseIsUp: () => {
        return new Promise((resolve, reject) => {
            pool.query('SELECT 1', (err, res) => {
                if (err) {
                    reject(false);
                } else {
                    resolve(true);
                }
            });
        });
    }
};

module.exports = database;
